package com.usky.system.service;

import com.usky.common.core.constants.Constants;
import com.usky.common.core.util.IpUtils;
import com.usky.common.core.util.ServletUtils;
import com.usky.common.redis.core.RedisHelper;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.BeanMapperUtils;
import com.usky.common.core.util.StringUtils;
import com.usky.system.domain.SysLogininfor;
import com.usky.system.domain.SysLogininforVO;
import com.usky.system.domain.SysUser;
import com.usky.system.domain.SysUserVO;
import com.usky.system.domain.constants.UserConstants;
import com.usky.system.model.LoginUser;
import com.usky.system.service.enums.UserStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 登录校验方法
 *
 * @author ruoyi
 */
@Component
public class SysLoginService {

    @Autowired
    private ISysLogininforService sysLogininforService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private RedisHelper redisHelper;

    /**
     * 登录
     */
    public LoginUser login(String username, String password, Integer tenantId) {
        // 用户名或密码为空 错误
        if (StringUtils.isAnyBlank(username, password)) {
            recordLogininfor(username, Constants.LOGIN_FAIL, "用户/密码必须填写");
            throw new BusinessException("用户/密码必须填写");
        }
        // 密码如果不在指定范围内 错误
        if (password.length() < UserConstants.PASSWORD_MIN_LENGTH
                || password.length() > UserConstants.PASSWORD_MAX_LENGTH) {
            recordLogininfor(username, Constants.LOGIN_FAIL, "用户密码不在指定范围");
            throw new BusinessException("用户密码不在指定范围");
        }
        // 用户名不在指定范围内 错误
        if (username.length() < UserConstants.USERNAME_MIN_LENGTH
                || username.length() > UserConstants.USERNAME_MAX_LENGTH) {
            recordLogininfor(username, Constants.LOGIN_FAIL, "用户名不在指定范围");
            throw new BusinessException("用户名不在指定范围");
        }

        // 查询用户信息
        LoginUser loginUser = new LoginUser();
        if (tenantId != null && tenantId != 0) {
            loginUser = sysUserService.getUserInfoOne(username, tenantId);
        } else {
            loginUser = sysUserService.getUserInfo(username);
        }

        SysUserVO user = loginUser.getSysUser();
        if (UserStatus.DELETED.getCode().equals(user.getDelFlag())) {
            recordLogininfor(username, Constants.LOGIN_FAIL, "对不起，您的账号已被删除");
            throw new BusinessException("对不起，您的账号：" + username + " 已被删除");
        }
        if (UserStatus.DISABLE.getCode().equals(user.getStatus())) {
            recordLogininfor(username, Constants.LOGIN_FAIL, "用户已停用，请联系管理员");
            throw new BusinessException("对不起，您的账号：" + username + " 已停用");
        }
        if (!SecurityUtils.matchesPassword(password, user.getPassword())) {
            recordLogininfor(username, Constants.LOGIN_FAIL, "用户密码错误");
            throw new BusinessException("用户不存在/密码错误");
        }
        recordLogininfor(username, Constants.LOGIN_SUCCESS, "登录成功");
        return loginUser;
    }


    public SysUser appLogin(String username, String password, Integer tenantId, String phone,String verify) {
        SysUser loginUser = new SysUser();
        if (!StringUtils.isBlank(username)) {
            // 用户名或密码为空 错误
            if (StringUtils.isAnyBlank(username, password)) {
                recordLogininfor(username, Constants.LOGIN_FAIL, "用户/密码必须填写");
                throw new BusinessException("用户/密码必须填写");
            }

            // 密码如果不在指定范围内 错误
            if (password.length() < UserConstants.PASSWORD_MIN_LENGTH
                    || password.length() > UserConstants.PASSWORD_MAX_LENGTH) {
                recordLogininfor(username, Constants.LOGIN_FAIL, "用户密码不在指定范围");
                throw new BusinessException("用户密码不在指定范围");
            }

            // 用户名不在指定范围内 错误
            if (username.length() < UserConstants.USERNAME_MIN_LENGTH
                    || username.length() > UserConstants.USERNAME_MAX_LENGTH) {
                recordLogininfor(username, Constants.LOGIN_FAIL, "用户名不在指定范围");
                throw new BusinessException("用户名不在指定范围");
            }

            //查询用户信息
            loginUser = sysUserService.getAppUserInfo(username, tenantId,null);

            if (!SecurityUtils.matchesPassword(password, loginUser.getPassword())) {
                recordLogininfor(username, Constants.LOGIN_FAIL, "用户密码错误");
                throw new BusinessException("用户不存在/密码错误");
            }
        } else {
            if (StringUtils.isBlank(verify)) {
                throw new BusinessException("验证码不能为空");
            }
            if (!verify.equals(redisHelper.get("Verify"))){
                throw new BusinessException("验证码错误");
            }
            if (StringUtils.isBlank(phone)){
                throw new BusinessException("用户名或手机号不能为空");
            }
            loginUser = sysUserService.getAppUserInfo(null, tenantId,phone);
        }

        if (UserStatus.DELETED.getCode().equals(loginUser.getDelFlag())) {
            recordLogininfor(username, Constants.LOGIN_FAIL, "对不起，您的账号已被删除");
            throw new BusinessException("对不起，您的账号：" + username + " 已被删除");
        }
        if (UserStatus.DISABLE.getCode().equals(loginUser.getStatus())) {
            recordLogininfor(username, Constants.LOGIN_FAIL, "用户已停用，请联系管理员");
            throw new BusinessException("对不起，您的账号：" + username + " 已停用");
        }

        recordLogininfor(username, Constants.LOGIN_SUCCESS, "登录成功");
        return loginUser;
    }


    public void logout(String loginName) {
        recordLogininfor(loginName, Constants.LOGOUT, "退出成功");
    }

    /**
     * 注册
     */
    public void register(String username, String password) {
        // 用户名或密码为空 错误
        if (StringUtils.isAnyBlank(username, password)) {
            throw new BusinessException("用户/密码必须填写");
        }
        if (username.length() < UserConstants.USERNAME_MIN_LENGTH
                || username.length() > UserConstants.USERNAME_MAX_LENGTH) {
            throw new BusinessException("账户长度必须在2到20个字符之间");
        }
        if (password.length() < UserConstants.PASSWORD_MIN_LENGTH
                || password.length() > UserConstants.PASSWORD_MAX_LENGTH) {
            throw new BusinessException("密码长度必须在5到20个字符之间");
        }

        // 注册用户信息
        SysUserVO sysUser = new SysUserVO();
        sysUser.setUserName(username);
        sysUser.setNickName(username);
        sysUser.setPassword(SecurityUtils.encryptPassword(password));
        sysUserService.register(BeanMapperUtils.map(sysUser, SysUser.class));
        recordLogininfor(username, Constants.REGISTER, "注册成功");
    }

    /**
     * 记录登录信息
     *
     * @param username 用户名
     * @param status   状态
     * @param message  消息内容
     * @return
     */
    public void recordLogininfor(String username, String status, String message) {
        SysLogininforVO logininfor = new SysLogininforVO();
        logininfor.setUserName(username);
        logininfor.setIpaddr(IpUtils.getIpAddr(ServletUtils.getRequest()));
        logininfor.setMsg(message);
        // 日志状态
        if (StringUtils.equalsAny(status, Constants.LOGIN_SUCCESS, Constants.LOGOUT, Constants.REGISTER)) {
            logininfor.setStatus(Constants.LOGIN_SUCCESS_STATUS);
        } else if (Constants.LOGIN_FAIL.equals(status)) {
            logininfor.setStatus(Constants.LOGIN_FAIL_STATUS);
        }
        sysLogininforService.insertLogininfor(BeanMapperUtils.map(logininfor, SysLogininfor.class));
    }


}